//Michael Good, 2/17/2012
//Chat Server - Manages texting session between multiple users over the network/internet

import java.net.*;
import java.io.*;

public class ChatServer
{
	protected static Socket client;
	protected static ServerSocket host;
	protected boolean exit = false;
	protected String output;
	protected ChatClient[] clientManager;
	protected Thread[] clientProcess;

	//Setup/Manage server and chat clients
	public ChatServer()
	{
		try
		{
			host	= new ServerSocket(7005);
		}
		catch(IOException e)
		{
			System.out.println("ERROR: Bad socket");
			System.exit(1);
		}
		
		clientManager = new ChatClient[5];
		clientProcess = new Thread[5];
		int nextOpen = -1;
		int ctr = 0;
		
		//manage chat clients
		while (true)
		{
			try
			{
				while (nextOpen == -1)
				{
					if (clientManager[ctr] == null)
						nextOpen = ctr;
						
					ctr++;
					
					if (ctr == clientManager.length)
						ctr = 0;
				}
				
				if (nextOpen == clientManager.length - 1)
					System.out.println("WARNING: Server may be reaching capacity");
				
				clientManager[nextOpen] = new ChatClient(host.accept(), nextOpen);
				clientProcess[nextOpen] = new Thread(clientManager[nextOpen]);
				clientProcess[nextOpen].start();
			}
			catch(IOException e)
			{
				System.out.println("ERROR: Client manager IO");
				System.exit(1);
			}
			
			nextOpen = -1;
		}
	}
	
	//Sends a new text to all the clients
	private void sendAll(String toSend)
	{
		for (int ctr = 0; ctr < clientManager.length; ctr++)
		{
			if (clientManager[ctr] != null)
			{
				clientManager[ctr].send(toSend);
			}
		}
	}
	
	//Resets a client so new connections can be established
	private void stopClient(int id)
	{
		clientManager[id] = null;
		clientProcess[id] = null;
		
		System.out.println("connection " + id + " terminated");
	}
	
	//Thread checks for and sends new texts to client
	private class ChatClient implements Runnable
	{
		protected PrintWriter out;
		protected BufferedReader in;
		private Socket client;
		protected int id;
	
		//Initialization
		public ChatClient(Socket C, int I)
		{
			client = C;
			id = I;
			
			System.out.println("new connection " + id + " established");
		}
		
		//Check for new texts from client
		public void run()
		{
			String get;
			boolean goodStream = true;
		
			try
			{
				out = new PrintWriter(client.getOutputStream(), true);
				in = new BufferedReader(new InputStreamReader(client.getInputStream()));
			}
			catch(IOException e)
			{
				System.out.println("ERROR: Bad connection");
				stopClient(id);
				goodStream = false;
			}
			
			while(goodStream){
     			try
				{
        			get = in.readLine();
					sendAll(get);
       		} 
				catch (IOException e) 
				{
         		stopClient(id);
					goodStream = false;
       		}
    		}
		}
		
		//Send new texts back to client
		public void send(String toSend)
		{
			out.println(toSend);
		}
	}
	
	//Initialize program
	public static void main(String[] Args)
	{
		ChatServer program = new ChatServer();
	}
}